# coding: utf-8
from django.shortcuts import render, redirect
from .models import Auditor
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.conf import settings

def admin(request, template_name='auditor/admin.html'):

    auditor_list = Auditor.objects.all()

    paginator = Paginator(auditor_list, 15)

    page = request.GET.get('page')
    try:
        audits = paginator.page(page)
    except PageNotAnInteger:
        audits = paginator.page(1)
    except EmptyPage:
        audits = paginator.page(paginator.num_pages)

    if 'tenant' in settings.AUDITOR:
        tenant = True
    else:
        tenant = False
    return render(request, template_name, {"audits": audits, "tenant": tenant})


# coding: utf-8

from django.db import models
from django.conf import settings
from django.contrib.contenttypes.models import ContentType

class Auditor(models.Model):
	field = models.CharField(max_length=255, blank=True)
	action = models.CharField(max_length=6)
	old_value = models.TextField(blank=True, null=True)
	new_value = models.TextField(blank=True, null=True)
	stamp = models.DateTimeField(auto_now_add=True)
	user = models.CharField(max_length=255)
	content_type = models.ForeignKey(ContentType, blank=True)
	object_id = models.PositiveIntegerField(blank=True)
	content_object = models.TextField()
	if 'tenant' in settings.AUDITOR:
		tenant = models.ForeignKey(settings.AUDITOR['tenant'])

	class Meta:
		ordering = ['-id']
		db_table = 'auditor'


# coding: utf-8

from .models import Auditor
from django.contrib.contenttypes.models import ContentType
from copy import deepcopy
from django.conf import settings

class Audit:
    def __init__(self, request, obj):
        self.original_object = deepcopy(obj)
        self.modified_object = obj
        self.user = str(request.user)
        self.content_type = ContentType.objects.get_for_model(
            self.original_object)
        if 'tenant' in settings.AUDITOR:
            self.tenant = request.tenant
        else:
            self.tenant = None

    def create_audit(self, action='', field=False):
        audit = Auditor(content_type=self.content_type)
        audit.object_id = self.original_object.pk
        audit.user = self.user
        audit.action = action
        audit.content_object = self.original_object
        audit.tenant = self.tenant
        if field:
            audit.field = field
        audit.save()

    def create(self):
        self.create_audit(action='CREATE')
        
    def delete(self):
        self.create_audit(action='DELETE')

    def audit_field(self, action='', field=''):
        """
        Pass an action and a field to register a 
        particular change
        """
        self.create_audit(action=action, field=field)
        
    def update(self, modified_object=False):
        """
        Although the modified_object argument is optional,
        you may want to pass the modified object
        """
        if modified_object:
            self.modified_object = modified_object

        # Loop through object fields
        for field in self.original_object._meta.get_fields():

            # Try to get the field value
            try:
                original_value = getattr(self.original_object, field.name)
                modified_value = getattr(self.modified_object, field.name)

                # Don't audit related fields in order to avoid duplicated data
                if str(field.name)[-3:] == '_id':
                    audit_field = False
                else:
                    audit_field = True

            except:
                audit_field = False

            if (audit_field and
                field.name != 'id' and
                (original_value != modified_value) and not
                ((original_value is None and modified_value == '') or
                    (original_value == '' and modified_value is None))):
                """
                Check if fields values are different (id field excluded)
                Don't audit if value change from None to '' or '' to None
                """

                audit = Auditor(content_type=self.content_type)
                audit.object_id = self.modified_object.pk
                audit.user = self.user
                audit.action = "UPDATE"

                try:
                    audit.field = self.original_object._meta.get_field(field.name).verbose_name
                except:
                    continue

                audit.content_object = self.modified_object

                # Store the choice name, otherwise store original value
                if (hasattr(self.original_object, 'get_' + str(field.name) + '_display') and
                    hasattr(self.modified_object, 'get_' + str(field.name) + '_display')):

                    audit.old_value = getattr(self.original_object, 'get_%s_display' % field.name)()
                    audit.new_value = getattr(self.modified_object, 'get_%s_display' % field.name)()
                else:
                    audit.old_value = original_value
                    audit.new_value = modified_value

                audit.tenant = self.tenant

                audit.save()



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-auditor',
    version='1.0',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description='This Django app logs changes of models objects in a simple and granular way.',
    long_description=README,
    url='https://github.com/brunobastos/django-auditor',
    author='Bruno Bastos',
    author_email='zerobruno@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries',
        'Topic :: System :: Logging',
    ],
    keywords='django log audit',
)